@@ -74,6 +74,7 @@ CTracer_init(CTracer *self, PyObject *args_unused, PyObject *kwds_unused)
     self->cur_entry.last_line = -1;
 
     self->context = Py_None;
+    Py_INCREF(self->context);
 
     ret = RET_OK;
     goto ok;
@@ -393,7 +394,7 @@ CTracer_handle_call(CTracer *self, PyFrameObject *frame)
         if (PyErr_Occurred()) {
             goto error;
         }
-        STATS( self->stats.new_files++; )
+        STATS( self->stats.files++; )
 
         /* We've never considered this file before. */
         /* Ask should_trace about it. */
@@ -474,7 +475,7 @@ CTracer_handle_call(CTracer *self, PyFrameObject *frame)
                     if (PyErr_Occurred()) {
                         goto error;
                     }
-                    STATS( self->stats.new_files++; )
+                    STATS( self->stats.files++; )
                     STATS( self->stats.pycalls++; )
                     should_include_bool = PyObject_CallFunctionObjArgs(self->check_include, tracename, frame, NULL);
                     if (should_include_bool == NULL) {
@@ -824,6 +825,10 @@ CTracer_trace(CTracer *self, PyFrameObject *frame, int what, PyObject *arg_unuse
 {
     int ret = RET_ERROR;
 
+    #if DO_NOTHING
+    return RET_OK;
+    #endif
+
     #if WHAT_LOG || TRACE_LOG
     PyObject * ascii = NULL;
     #endif
@@ -922,6 +927,10 @@ CTracer_call(CTracer *self, PyObject *args, PyObject *kwds)
     PyObject *ret = NULL;
     PyObject * ascii = NULL;
 
+    #if DO_NOTHING
+    CRASH
+    #endif
+
     static char *what_names[] = {
         "call", "exception", "line", "return",
         "c_call", "c_exception", "c_return",
@@ -1032,7 +1041,7 @@ CTracer_get_stats(CTracer *self)
         "returns", self->stats.returns,
         "exceptions", self->stats.exceptions,
         "others", self->stats.others,
-        "new_files", self->stats.new_files,
+        "files", self->stats.files,
         "missed_returns", self->stats.missed_returns,
         "stack_reallocs", self->stats.stack_reallocs,
         "stack_alloc", self->pdata_stack->alloc,
