@@ -6,6 +6,7 @@
 import contextlib
 import datetime
 import glob
+import itertools
 import os
 import random
 import re
@@ -19,8 +20,10 @@ from unittest_mixins import (
 )
 
 import coverage
+from coverage import env
 from coverage.backunittest import TestCase
 from coverage.backward import StringIO, import_local_file, string_class, shlex_quote
+from coverage.backward import invalidate_import_caches
 from coverage.cmdline import CoverageScript
 from coverage.debug import _TEST_NAME_FILE, DebugControl
 
@@ -72,11 +75,13 @@ class CoverageTest(
         # Also have to clean out the .pyc file, since the timestamp
         # resolution is only one second, a changed file might not be
         # picked up.
-        for pyc in glob.glob('*.pyc'):
+        for pyc in itertools.chain(glob.glob('*.pyc'), glob.glob('*$py.class')):
             os.remove(pyc)
         if os.path.exists("__pycache__"):
             shutil.rmtree("__pycache__")
 
+        invalidate_import_caches()
+
     def import_local_file(self, modname, modfile=None):
         """Import a local file as a module.
 
@@ -237,17 +242,17 @@ class CoverageTest(
             with self.delayed_assertions():
                 self.assert_equal_args(
                     analysis.arc_possibilities(), arcs,
-                    "Possible arcs differ",
+                    "Possible arcs differ: minus is actual, plus is expected"
                 )
 
                 self.assert_equal_args(
                     analysis.arcs_missing(), arcs_missing,
-                    "Missing arcs differ"
+                    "Missing arcs differ: minus is actual, plus is expected"
                 )
 
                 self.assert_equal_args(
                     analysis.arcs_unpredicted(), arcs_unpredicted,
-                    "Unpredicted arcs differ"
+                    "Unpredicted arcs differ: minus is actual, plus is expected"
                 )
 
         if report:
@@ -341,39 +346,14 @@ class CoverageTest(
         combined content of `stdout` and `stderr` output streams from the
         sub-process.
 
+        See `run_command_status` for complete semantics.
+
         Use this when you need to test the process behavior of coverage.
 
         Compare with `command_line`.
 
-        Handles the following command name specially:
-
-        * "python" is replaced with the command name of the current
-            Python interpreter.
-
-        * "coverage" is replaced with the command name for the main
-            Coverage.py program.
-
         """
-        split_commandline = cmd.split(" ", 1)
-        command_name = split_commandline[0]
-        command_args = split_commandline[1:]
-
-        if command_name == "python":
-            # Running a Python interpreter in a sub-processes can be tricky.
-            # Use the real name of our own executable. So "python foo.py" might
-            # get executed as "python3.3 foo.py". This is important because
-            # Python 3.x doesn't install as "python", so you might get a Python
-            # 2 executable instead if you don't use the executable's basename.
-            command_name = os.path.basename(sys.executable)
-
-        if command_name == "coverage":
-            # The invocation requests the Coverage.py program.  Substitute the
-            # actual Coverage.py main command name.
-            command_name = self.coverage_command
-
-        full_commandline = " ".join([shlex_quote(command_name)] + command_args)
-
-        _, output = self.run_command_status(full_commandline)
+        _, output = self.run_command_status(cmd)
         return output
 
     def run_command_status(self, cmd):
@@ -383,25 +363,74 @@ class CoverageTest(
 
         Compare with `command_line`.
 
-        Returns a pair: the process' exit status and stdout text, which are
-        also stored as self.last_command_status and self.last_command_output.
+        Handles the following command names specially:
+
+        * "python" is replaced with the command name of the current
+            Python interpreter.
+
+        * "coverage" is replaced with the command name for the main
+            Coverage.py program.
+
+        Returns a pair: the process' exit status and its stdout/stderr text,
+        which are also stored as `self.last_command_status` and
+        `self.last_command_output`.
 
         """
+        # Make sure "python" and "coverage" mean specifically what we want
+        # them to mean.
+        split_commandline = cmd.split()
+        command_name = split_commandline[0]
+        command_args = split_commandline[1:]
+
+        if command_name == "python":
+            # Running a Python interpreter in a sub-processes can be tricky.
+            # Use the real name of our own executable. So "python foo.py" might
+            # get executed as "python3.3 foo.py". This is important because
+            # Python 3.x doesn't install as "python", so you might get a Python
+            # 2 executable instead if you don't use the executable's basename.
+            command_words = [os.path.basename(sys.executable)]
+
+        elif command_name == "coverage":
+            if env.JYTHON:                  # pragma: only jython
+                # Jython can't do reporting, so let's skip the test now.
+                if command_args and command_args[0] in ('report', 'html', 'xml', 'annotate'):
+                    self.skipTest("Can't run reporting commands in Jython")
+                # Jython can't run "coverage" as a command because the shebang
+                # refers to another shebang'd Python script. So run them as
+                # modules.
+                command_words = "jython -m coverage".split()
+            else:
+                # The invocation requests the Coverage.py program.  Substitute the
+                # actual Coverage.py main command name.
+                command_words = [self.coverage_command]
+
+        else:
+            command_words = [command_name]
+
+        cmd = " ".join([shlex_quote(w) for w in command_words] + command_args)
+
         # Add our test modules directory to PYTHONPATH.  I'm sure there's too
         # much path munging here, but...
-        here = os.path.dirname(self.nice_file(coverage.__file__, ".."))
-        testmods = self.nice_file(here, 'tests/modules')
-        zipfile = self.nice_file(here, 'tests/zipmods.zip')
-        pypath = os.getenv('PYTHONPATH', '')
+        pythonpath_name = "PYTHONPATH"
+        if env.JYTHON:
+            pythonpath_name = "JYTHONPATH"          # pragma: only jython
+
+        testmods = self.nice_file(self.working_root(), 'tests/modules')
+        zipfile = self.nice_file(self.working_root(), 'tests/zipmods.zip')
+        pypath = os.getenv(pythonpath_name, '')
         if pypath:
             pypath += os.pathsep
         pypath += testmods + os.pathsep + zipfile
-        self.set_environ('PYTHONPATH', pypath)
+        self.set_environ(pythonpath_name, pypath)
 
         self.last_command_status, self.last_command_output = run_command(cmd)
         print(self.last_command_output)
         return self.last_command_status, self.last_command_output
 
+    def working_root(self):
+        """Where is the root of the coverage.py working tree?"""
+        return os.path.dirname(self.nice_file(coverage.__file__, ".."))
+
     def report_from_command(self, cmd):
         """Return the report from the `cmd`, with some convenience added."""
         report = self.run_command(cmd).replace('\\', '/')
