@@ -35,7 +35,7 @@ class ApiTest(CoverageTest):
     def assertFiles(self, files):
         """Assert that the files here are `files`, ignoring the usual junk."""
         here = os.listdir(".")
-        here = self.clean_files(here, ["*.pyc", "__pycache__"])
+        here = self.clean_files(here, ["*.pyc", "__pycache__", "*$py.class"])
         self.assertCountEqual(here, files)
 
     def test_unexecuted_file(self):
@@ -281,7 +281,7 @@ class ApiTest(CoverageTest):
         self.start_import_stop(cov, "code2")
         self.check_code1_code2(cov)
 
-    def test_start_save_stop(self):
+    def test_start_save_stop(self):             # pragma: not covered
         self.skipTest("Expected failure: https://bitbucket.org/ned/coveragepy/issue/79")
         self.make_code1_code2()
         cov = coverage.Coverage()
@@ -293,8 +293,8 @@ class ApiTest(CoverageTest):
 
         self.check_code1_code2(cov)
 
-    def make_corrupt_data_files(self):
-        """Make some good and some bad data files."""
+    def make_good_data_files(self):
+        """Make some good data files."""
         self.make_code1_code2()
         cov = coverage.Coverage(data_suffix=True)
         self.start_import_stop(cov, "code1")
@@ -304,12 +304,15 @@ class ApiTest(CoverageTest):
         self.start_import_stop(cov, "code2")
         cov.save()
 
+    def make_bad_data_file(self):
+        """Make one bad data file."""
         self.make_file(".coverage.foo", """La la la, this isn't coverage data!""")
 
     def test_combining_corrupt_data(self):
         # If you combine a corrupt data file, then you will get a warning,
         # and the file will remain.
-        self.make_corrupt_data_files()
+        self.make_good_data_files()
+        self.make_bad_data_file()
         cov = coverage.Coverage()
         warning_regex = (
             r"Couldn't read data from '.*\.coverage\.foo': "
@@ -324,6 +327,27 @@ class ApiTest(CoverageTest):
         # The bad file still exists.
         self.assert_exists(".coverage.foo")
 
+    def test_combining_twice(self):
+        self.make_good_data_files()
+        cov1 = coverage.Coverage()
+        cov1.combine()
+        cov1.save()
+        self.check_code1_code2(cov1)
+
+        cov2 = coverage.Coverage()
+        with self.assertRaisesRegex(CoverageException, r"No data to combine"):
+            cov2.combine(strict=True)
+
+        cov3 = coverage.Coverage()
+        cov3.combine()
+        # Now the data is empty!
+        _, statements, missing, _ = cov3.analysis("code1.py")
+        self.assertEqual(statements, [1])
+        self.assertEqual(missing, [1])
+        _, statements, missing, _ = cov3.analysis("code2.py")
+        self.assertEqual(statements, [1, 2])
+        self.assertEqual(missing, [1, 2])
+
 
 class NamespaceModuleTest(CoverageTest):
     """Test PEP-420 namespace modules."""
@@ -344,7 +368,6 @@ class NamespaceModuleTest(CoverageTest):
             cov.analysis(sys.modules['namespace'])
 
 
-
 class UsingModulesMixin(object):
     """A mixin for importing modules from test/modules and test/moremodules."""
 
@@ -429,7 +452,7 @@ class SourceOmitIncludeTest(OmitIncludeTestsMixin, CoverageTest):
         """
         cov = coverage.Coverage(**kwargs)
         cov.start()
-        import usepkgs  # pragma: nested   # pylint: disable=import-error
+        import usepkgs  # pragma: nested   # pylint: disable=import-error, unused-variable
         cov.stop()      # pragma: nested
         data = cov.get_data()
         summary = data.line_counts()
@@ -438,13 +461,20 @@ class SourceOmitIncludeTest(OmitIncludeTestsMixin, CoverageTest):
             summary[k[:-3]] = v
         return summary
 
-    def test_source_package(self):
+    def test_source_package_as_dir(self):
+        # pkg1 is a directory, since we cd'd into tests/modules in setUp.
         lines = self.coverage_usepkgs(source=["pkg1"])
         self.filenames_in(lines, "p1a p1b")
         self.filenames_not_in(lines, "p2a p2b othera otherb osa osb")
         # Because source= was specified, we do search for unexecuted files.
         self.assertEqual(lines['p1c'], 0)
 
+    def test_source_package_as_package(self):
+        lines = self.coverage_usepkgs(source=["pkg1.sub"])
+        self.filenames_not_in(lines, "p2a p2b othera otherb osa osb")
+        # Because source= was specified, we do search for unexecuted files.
+        self.assertEqual(lines['runmod3'], 0)
+
     def test_source_package_dotted(self):
         lines = self.coverage_usepkgs(source=["pkg1.p1b"])
         self.filenames_in(lines, "p1b")
@@ -468,7 +498,7 @@ class ReportIncludeOmitTest(OmitIncludeTestsMixin, CoverageTest):
         """Try coverage.report()."""
         cov = coverage.Coverage()
         cov.start()
-        import usepkgs  # pragma: nested   # pylint: disable=import-error
+        import usepkgs  # pragma: nested   # pylint: disable=import-error, unused-variable
         cov.stop()      # pragma: nested
         report = StringIO()
         cov.report(file=report, **kwargs)
@@ -487,7 +517,7 @@ class XmlIncludeOmitTest(OmitIncludeTestsMixin, CoverageTest):
         """Try coverage.xml_report()."""
         cov = coverage.Coverage()
         cov.start()
-        import usepkgs  # pragma: nested   # pylint: disable=import-error
+        import usepkgs  # pragma: nested   # pylint: disable=import-error, unused-variable
         cov.stop()      # pragma: nested
         cov.xml_report(outfile="-", **kwargs)
         return self.stdout()
