@@ -252,12 +252,12 @@ class LoopArcTest(CoverageTest):
             """,
             arcz=".1 12 23 34 45 36 63 57 7.",
             )
-        # With "while True", 2.x thinks it's computation, 3.x thinks it's
-        # constant.
+        # With "while True", 2.x thinks it's computation,
+        # 3.x thinks it's constant.
         if env.PY3:
             arcz = ".1 12 23 34 45 36 63 57 7."
         else:
-            arcz = ".1 12 23 27 34 45 36 62 57 7."
+            arcz = ".1 12 23 34 45 36 62 57 7."
         self.check_coverage("""\
             a, i = 1, 0
             while True:
@@ -270,6 +270,37 @@ class LoopArcTest(CoverageTest):
             arcz=arcz,
         )
 
+    def test_zero_coverage_while_loop(self):
+        # https://bitbucket.org/ned/coveragepy/issue/502
+        self.make_file("main.py", "print('done')")
+        self.make_file("zero.py", """\
+            def method(self):
+                while True:
+                    return 1
+            """)
+        out = self.run_command("coverage run --branch --source=. main.py")
+        self.assertEqual(out, 'done\n')
+        report = self.report_from_command("coverage report -m")
+        squeezed = self.squeezed_lines(report)
+        self.assertIn("zero.py 3 3 0 0 0% 1-3", squeezed[3])
+
+    def test_bug_496_continue_in_constant_while(self):
+        # https://bitbucket.org/ned/coveragepy/issue/496
+        if env.PY3:
+            arcz = ".1 12 23 34 45 53 46 6."
+        else:
+            arcz = ".1 12 23 34 45 52 46 6."
+        self.check_coverage("""\
+            up = iter('ta')
+            while True:
+                char = next(up)
+                if char == 't':
+                    continue
+                break
+            """,
+            arcz=arcz
+            )
+
     def test_for_if_else_for(self):
         self.check_coverage("""\
             def branches_2(l):
@@ -370,7 +401,7 @@ class LoopArcTest(CoverageTest):
 
     def test_other_comprehensions(self):
         if env.PYVERSION < (2, 7):
-            self.skipTest("Don't have set or dict comprehensions before 2.7")
+            self.skipTest("No set or dict comprehensions before 2.7")       # pragma: not covered
         # Set comprehension:
         self.check_coverage("""\
             o = ((1,2), (3,4))
@@ -394,7 +425,7 @@ class LoopArcTest(CoverageTest):
 
     def test_multiline_dict_comp(self):
         if env.PYVERSION < (2, 7):
-            self.skipTest("Don't have set or dict comprehensions before 2.7")
+            self.skipTest("No set or dict comprehensions before 2.7")       # pragma: not covered
         if env.PYVERSION < (3, 5):
             arcz = "-42 2B B-4   2-4"
         else:
@@ -762,16 +793,19 @@ class ExceptionArcTest(CoverageTest):
     def test_return_finally(self):
         self.check_coverage("""\
             a = [1]
-            def func():
-                try:
-                    return 10
-                finally:
-                    a.append(6)
-
-            assert func() == 10
-            assert a == [1, 6]
+            def check_token(data):
+                if data:
+                    try:
+                        return 5
+                    finally:
+                        a.append(7)
+                return 8
+            assert check_token(False) == 8
+            assert a == [1]
+            assert check_token(True) == 5
+            assert a == [1, 7]
             """,
-            arcz=".1 12 28 89 9.  -23 34 46 6-2",
+            arcz=".1 12 29 9A AB BC C-1   -23 34 45 57 7-2 38 8-2",
         )
 
     def test_except_jump_finally(self):
@@ -998,6 +1032,102 @@ class YieldTest(CoverageTest):
         )
 
 
+class OptimizedIfTest(CoverageTest):
+    """Tests of if statements being optimized away."""
+
+    def test_optimized_away_if_0(self):
+        self.check_coverage("""\
+            a = 1
+            if len([2]):
+                c = 3
+            if 0:               # this line isn't in the compiled code.
+                if len([5]):
+                    d = 6
+            else:
+                e = 8
+            f = 9
+            """,
+            lines=[1, 2, 3, 8, 9],
+            arcz=".1 12 23 28 38 89 9.",
+            arcz_missing="28",
+        )
+
+    def test_optimized_away_if_1(self):
+        self.check_coverage("""\
+            a = 1
+            if len([2]):
+                c = 3
+            if 1:               # this line isn't in the compiled code,
+                if len([5]):    # but these are.
+                    d = 6
+            else:
+                e = 8
+            f = 9
+            """,
+            lines=[1, 2, 3, 5, 6, 9],
+            arcz=".1 12 23 25 35 56 69 59 9.",
+            arcz_missing="25 59",
+        )
+        self.check_coverage("""\
+            a = 1
+            if 1:
+                b = 3
+                c = 4
+            d = 5
+            """,
+            lines=[1, 3, 4, 5],
+            arcz=".1 13 34 45 5.",
+        )
+
+    def test_optimized_nested(self):
+        self.check_coverage("""\
+            a = 1
+            if 0:
+                if 0:
+                    b = 4
+                else:
+                    c = 6
+            else:
+                if 0:
+                    d = 9
+                else:
+                    if 0: e = 11
+                    f = 12
+                    if 0: g = 13
+            h = 14
+            """,
+            lines=[1, 12, 14],
+            arcz=".1 1C CE E.",
+        )
+
+    def test_constant_if(self):
+        if env.PYPY:
+            self.skipTest("PyPy doesn't optimize away 'if __debug__:'")
+        # CPython optimizes away "if __debug__:"
+        self.check_coverage("""\
+            for value in [True, False]:
+                if value:
+                    if __debug__:
+                        x = 4
+                else:
+                    x = 6
+            """,
+            arcz=".1 12 24 41 26 61 1.",
+        )
+        # No Python optimizes away "if not __debug__:"
+        self.check_coverage("""\
+            for value in [True, False]:
+                if value:
+                    if not __debug__:
+                        x = 4
+                else:
+                    x = 6
+            """,
+            arcz=".1 12 23 31 34 41 26 61 1.",
+            arcz_missing="34 41",
+        )
+
+
 class MiscArcTest(CoverageTest):
     """Miscellaneous arc-measuring tests."""
 
@@ -1067,6 +1197,9 @@ class MiscArcTest(CoverageTest):
         )
 
     def test_pathologically_long_code_object(self):
+        if env.JYTHON:
+            self.skipTest("Bytecode concerns are irrelevant on Jython")
+
         # https://bitbucket.org/ned/coveragepy/issue/359
         # The structure of this file is such that an EXTENDED_ARG bytecode is
         # needed to encode the jump at the end.  We weren't interpreting those
@@ -1090,21 +1223,6 @@ class MiscArcTest(CoverageTest):
             arcs_missing=[], arcs_unpredicted=[],
             )
 
-    def test_optimized_away_lines(self):
-        self.check_coverage("""\
-            a = 1
-            if len([2]):
-                c = 3
-            if 0:               # this line isn't in the compiled code.
-                if len([5]):
-                    d = 6
-            e = 7
-            """,
-            lines=[1, 2, 3, 7],
-            arcz=".1 12 23 27 37 7.",
-            arcz_missing="27",
-        )
-
     def test_partial_generators(self):
         # https://bitbucket.org/ned/coveragepy/issues/475/generator-expression-is-marked-as-not
         # Line 2 is executed completely.
@@ -1340,7 +1458,7 @@ class AsyncTest(CoverageTest):
                 def __init__(self, obj):                # 4
                     self._it = iter(obj)
 
-                async def __aiter__(self):              # 7
+                def __aiter__(self):                    # 7
                     return self
 
                 async def __anext__(self):              # A
