@@ -10,6 +10,7 @@ import os.path
 import re
 import sys
 
+from coverage import env
 from coverage.backward import binary_bytes
 from coverage.execfile import run_python_file, run_python_module
 from coverage.misc import NoCode, NoSource
@@ -102,6 +103,9 @@ class RunPycFileTest(CoverageTest):
 
     def make_pyc(self):
         """Create a .pyc file, and return the relative path to it."""
+        if env.JYTHON:
+            self.skipTest("Can't make .pyc files on Jython")
+
         self.make_file("compiled.py", """\
             def doit():
                 print("I am here!")
@@ -145,6 +149,25 @@ class RunPycFileTest(CoverageTest):
         with self.assertRaisesRegex(NoCode, "No file to run: 'xyzzy.pyc'"):
             run_python_file("xyzzy.pyc", [])
 
+    def test_running_py_from_binary(self):
+        # Use make_file to get the bookkeeping. Ideally, it would
+        # be able to write binary files.
+        bf = self.make_file("binary")
+        with open(bf, "wb") as f:
+            f.write(b'\x7fELF\x02\x01\x01\x00\x00\x00')
+
+        msg = (
+            r"Couldn't run 'binary' as Python code: "
+            r"(TypeError|ValueError): "
+            r"("
+            r"compile\(\) expected string without null bytes"    # for py2
+            r"|"
+            r"source code string cannot contain null bytes"     # for py3
+            r")"
+        )
+        with self.assertRaisesRegex(Exception, msg):
+            run_python_file(bf, [bf])
+
 
 class RunModuleTest(CoverageTest):
     """Test run_python_module."""
