@@ -5,6 +5,7 @@
 
 import os.path
 import re
+import textwrap
 
 from coverage import env
 from coverage.phystokens import source_token_lines, source_encoding
@@ -14,18 +15,35 @@ from coverage.python import get_python_source
 from tests.coveragetest import CoverageTest
 
 
+# A simple program and its token stream.
 SIMPLE = u"""\
 # yay!
 def foo():
   say('two = %d' % 2)
 """
 
+SIMPLE_TOKENS = [
+    [('com', "# yay!")],
+    [('key', 'def'), ('ws', ' '), ('nam', 'foo'), ('op', '('), ('op', ')'), ('op', ':')],
+    [('ws', '  '), ('nam', 'say'), ('op', '('),
+        ('str', "'two = %d'"), ('ws', ' '), ('op', '%'),
+        ('ws', ' '), ('num', '2'), ('op', ')')],
+]
+
+# Mixed-whitespace program, and its token stream.
 MIXED_WS = u"""\
 def hello():
         a="Hello world!"
 \tb="indented"
 """
 
+MIXED_WS_TOKENS = [
+    [('key', 'def'), ('ws', ' '), ('nam', 'hello'), ('op', '('), ('op', ')'), ('op', ':')],
+    [('ws', '        '), ('nam', 'a'), ('op', '='), ('str', '"Hello world!"')],
+    [('ws', '        '), ('nam', 'b'), ('op', '='), ('str', '"indented"')],
+]
+
+# Where this file is, so we can find other files next to it.
 HERE = os.path.dirname(__file__)
 
 
@@ -52,28 +70,16 @@ class PhysTokensTest(CoverageTest):
         self.check_tokenization(get_python_source(fname))
 
     def test_simple(self):
-        self.assertEqual(list(source_token_lines(SIMPLE)),
-            [
-                [('com', "# yay!")],
-                [('key', 'def'), ('ws', ' '), ('nam', 'foo'), ('op', '('),
-                            ('op', ')'), ('op', ':')],
-                [('ws', '  '), ('nam', 'say'), ('op', '('),
-                            ('str', "'two = %d'"), ('ws', ' '), ('op', '%'),
-                            ('ws', ' '), ('num', '2'), ('op', ')')]
-            ])
+        self.assertEqual(list(source_token_lines(SIMPLE)), SIMPLE_TOKENS)
         self.check_tokenization(SIMPLE)
 
+    def test_missing_final_newline(self):
+        # We can tokenize source that is missing the final newline.
+        self.assertEqual(list(source_token_lines(SIMPLE.rstrip())), SIMPLE_TOKENS)
+
     def test_tab_indentation(self):
         # Mixed tabs and spaces...
-        self.assertEqual(list(source_token_lines(MIXED_WS)),
-            [
-                [('key', 'def'), ('ws', ' '), ('nam', 'hello'), ('op', '('),
-                            ('op', ')'), ('op', ':')],
-                [('ws', '        '), ('nam', 'a'), ('op', '='),
-                            ('str', '"Hello world!"')],
-                [('ws', '        '), ('nam', 'b'), ('op', '='),
-                            ('str', '"indented"')],
-            ])
+        self.assertEqual(list(source_token_lines(MIXED_WS)), MIXED_WS_TOKENS)
 
     def test_tokenize_real_file(self):
         # Check the tokenization of a real file (large, btw).
@@ -97,13 +103,15 @@ else:
 
 ENCODING_DECLARATION_SOURCES = [
     # Various forms from http://www.python.org/dev/peps/pep-0263/
-    (1, b"# coding=cp850\n\n"),
-    (1, b"#!/usr/bin/python\n# -*- coding: cp850 -*-\n"),
-    (1, b"#!/usr/bin/python\n# vim: set fileencoding=cp850:\n"),
-    (1, b"# This Python file uses this encoding: cp850\n"),
-    (1, b"# This file uses a different encoding:\n# coding: cp850\n"),
-    (1, b"\n# coding=cp850\n\n"),
-    (2, b"# -*-  coding:cp850 -*-\n# vim: fileencoding=cp850\n"),
+    (1, b"# coding=cp850\n\n", "cp850"),
+    (1, b"# coding=latin-1\n", "iso-8859-1"),
+    (1, b"# coding=iso-latin-1\n", "iso-8859-1"),
+    (1, b"#!/usr/bin/python\n# -*- coding: cp850 -*-\n", "cp850"),
+    (1, b"#!/usr/bin/python\n# vim: set fileencoding=cp850:\n", "cp850"),
+    (1, b"# This Python file uses this encoding: cp850\n", "cp850"),
+    (1, b"# This file uses a different encoding:\n# coding: cp850\n", "cp850"),
+    (1, b"\n# coding=cp850\n\n", "cp850"),
+    (2, b"# -*-  coding:cp850 -*-\n# vim: fileencoding=cp850\n", "cp850"),
 ]
 
 class SourceEncodingTest(CoverageTest):
@@ -112,10 +120,10 @@ class SourceEncodingTest(CoverageTest):
     run_in_temp_dir = False
 
     def test_detect_source_encoding(self):
-        for _, source in ENCODING_DECLARATION_SOURCES:
+        for _, source, expected in ENCODING_DECLARATION_SOURCES:
             self.assertEqual(
                 source_encoding(source),
-                'cp850',
+                expected,
                 "Wrong encoding in %r" % source
             )
 
@@ -123,7 +131,7 @@ class SourceEncodingTest(CoverageTest):
         if env.PYPY and env.PY3:
             # PyPy3 gets this case wrong. Not sure what I can do about it,
             # so skip the test.
-            self.skip("PyPy3 is wrong about non-comment encoding. Skip it.")
+            self.skipTest("PyPy3 is wrong about non-comment encoding. Skip it.")
         # Should not detect anything here
         source = b'def parse(src, encoding=None):\n    pass'
         self.assertEqual(source_encoding(source), DEF_ENCODING)
@@ -142,9 +150,19 @@ class SourceEncodingTest(CoverageTest):
         source = b"\xEF\xBB\xBFtext = 'hello'\n"
         self.assertEqual(source_encoding(source), 'utf-8-sig')
 
-        # But it has to be the only authority.
+    def test_bom_with_encoding(self):
+        source = b"\xEF\xBB\xBF# coding: utf-8\ntext = 'hello'\n"
+        self.assertEqual(source_encoding(source), 'utf-8-sig')
+
+    def test_bom_is_wrong(self):
+        # A BOM with an explicit non-utf8 encoding is an error.
         source = b"\xEF\xBB\xBF# coding: cp850\n"
-        with self.assertRaises(SyntaxError):
+        with self.assertRaisesRegex(SyntaxError, "encoding problem: utf-8"):
+            source_encoding(source)
+
+    def test_unknown_encoding(self):
+        source = b"# coding: klingon\n"
+        with self.assertRaisesRegex(SyntaxError, "unknown encoding: klingon"):
             source_encoding(source)
 
 
@@ -154,7 +172,7 @@ class NeuterEncodingDeclarationTest(CoverageTest):
     run_in_temp_dir = False
 
     def test_neuter_encoding_declaration(self):
-        for lines_diff_expected, source in ENCODING_DECLARATION_SOURCES:
+        for lines_diff_expected, source, _ in ENCODING_DECLARATION_SOURCES:
             neutered = neuter_encoding_declaration(source.decode("ascii"))
             neutered = neutered.encode("ascii")
 
@@ -177,6 +195,63 @@ class NeuterEncodingDeclarationTest(CoverageTest):
                 "Wrong encoding in %r" % neutered
             )
 
+    def test_two_encoding_declarations(self):
+        input_src = textwrap.dedent(u"""\
+            # -*- coding: ascii -*-
+            # -*- coding: utf-8 -*-
+            # -*- coding: utf-16 -*-
+            """)
+        expected_src = textwrap.dedent(u"""\
+            # (deleted declaration) -*-
+            # (deleted declaration) -*-
+            # -*- coding: utf-16 -*-
+            """)
+        output_src = neuter_encoding_declaration(input_src)
+        self.assertEqual(expected_src, output_src)
+
+    def test_one_encoding_declaration(self):
+        input_src = textwrap.dedent(u"""\
+            # -*- coding: utf-16 -*-
+            # Just a comment.
+            # -*- coding: ascii -*-
+            """)
+        expected_src = textwrap.dedent(u"""\
+            # (deleted declaration) -*-
+            # Just a comment.
+            # -*- coding: ascii -*-
+            """)
+        output_src = neuter_encoding_declaration(input_src)
+        self.assertEqual(expected_src, output_src)
+
+
+class Bug529Test(CoverageTest):
+    """Test of bug 529"""
+
+    def test_bug_529(self):
+        self.make_file("the_test.py", '''\
+            # -*- coding: utf-8 -*-
+            import unittest
+            class FailsUnderCoverageTest(unittest.TestCase):
+                def test_fails_under_coverage(self):
+                    src1 = u"""\\
+                        # -*- coding: utf-8 -*-
+                        # Just a comment.
+                        """
+                    src2 = u"""\\
+                        # -*- coding: utf-8 -*-
+                        # Just a comment.
+                        """
+                    self.assertEqual(src1, src2)
+
+            if __name__ == "__main__":
+                unittest.main()
+            ''')
+        status, out = self.run_command_status("coverage run the_test.py")
+        self.assertEqual(status, 0)
+        self.assertIn("OK", out)
+        # If this test fails, the output will be super-confusing, because it
+        # has a failing unit test contained within the failing unit test.
+
 
 class CompileUnicodeTest(CoverageTest):
     """Tests of compiling Unicode strings."""
