@@ -361,6 +361,27 @@ class SummaryTest(CoverageTest):
         squeezed = self.squeezed_lines(report)
         self.assertEqual(squeezed[3], "1 file skipped due to complete coverage.")
 
+    def test_report_skip_covered_longfilename(self):
+        self.make_file("long_______________filename.py", """
+            def foo():
+                pass
+            foo()
+        """)
+        out = self.run_command("coverage run --branch long_______________filename.py")
+        self.assertEqual(out, "")
+        report = self.report_from_command("coverage report --skip-covered")
+
+        # Name    Stmts   Miss Branch BrPart  Cover
+        # -----------------------------------------
+        #
+        # 1 file skipped due to complete coverage.
+
+        self.assertEqual(self.line_count(report), 4, report)
+        lines = self.report_lines(report)
+        self.assertEqual(lines[0], "Name    Stmts   Miss Branch BrPart  Cover")
+        squeezed = self.squeezed_lines(report)
+        self.assertEqual(squeezed[3], "1 file skipped due to complete coverage.")
+
     def test_report_skip_covered_no_data(self):
         report = self.report_from_command("coverage report --skip-covered")
 
@@ -381,18 +402,18 @@ class SummaryTest(CoverageTest):
         self.make_file("mycode.py", "This isn't python at all!")
         report = self.report_from_command("coverage report mycode.py")
 
+        # mycode   NotPython: Couldn't parse '...' as Python source: 'invalid syntax' at line 1
         # Name     Stmts   Miss  Cover
         # ----------------------------
-        # mycode   NotPython: Couldn't parse '...' as Python source: 'invalid syntax' at line 1
         # No data to report.
 
-        last = self.squeezed_lines(report)[-2]
+        errmsg = self.squeezed_lines(report)[0]
         # The actual file name varies run to run.
-        last = re.sub(r"parse '.*mycode.py", "parse 'mycode.py", last)
+        errmsg = re.sub(r"parse '.*mycode.py", "parse 'mycode.py", errmsg)
         # The actual error message varies version to version
-        last = re.sub(r": '.*' at", ": 'error' at", last)
+        errmsg = re.sub(r": '.*' at", ": 'error' at", errmsg)
         self.assertEqual(
-            last,
+            errmsg,
             "mycode.py NotPython: Couldn't parse 'mycode.py' as Python source: 'error' at line 1"
         )
 
@@ -405,16 +426,16 @@ class SummaryTest(CoverageTest):
         self.make_file(u"accented\xe2.py", "This isn't python at all!")
         report = self.report_from_command(u"coverage report accented\xe2.py")
 
+        # xxxx   NotPython: Couldn't parse '...' as Python source: 'invalid syntax' at line 1
         # Name     Stmts   Miss  Cover
         # ----------------------------
-        # xxxx   NotPython: Couldn't parse '...' as Python source: 'invalid syntax' at line 1
         # No data to report.
 
-        last = self.squeezed_lines(report)[-2]
+        errmsg = self.squeezed_lines(report)[0]
         # The actual file name varies run to run.
-        last = re.sub(r"parse '.*(accented.*?\.py)", r"parse '\1", last)
+        errmsg = re.sub(r"parse '.*(accented.*?\.py)", r"parse '\1", errmsg)
         # The actual error message varies version to version
-        last = re.sub(r": '.*' at", ": 'error' at", last)
+        errmsg = re.sub(r": '.*' at", ": 'error' at", errmsg)
         expected = (
             u"accented\xe2.py NotPython: "
             u"Couldn't parse 'accented\xe2.py' as Python source: 'error' at line 1"
@@ -422,7 +443,7 @@ class SummaryTest(CoverageTest):
         if env.PY2:
             # pylint: disable=redefined-variable-type
             expected = expected.encode(output_encoding())
-        self.assertEqual(last, expected)
+        self.assertEqual(errmsg, expected)
 
     def test_dotpy_not_python_ignored(self):
         # We run a .py file, and when reporting, we can't parse it as Python,
@@ -478,7 +499,7 @@ class SummaryTest(CoverageTest):
             """)
         cov = coverage.Coverage(branch=True, source=["."])
         cov.start()
-        import main     # pragma: nested # pylint: disable=import-error
+        import main     # pragma: nested # pylint: disable=import-error, unused-variable
         cov.stop()      # pragma: nested
         report = self.get_report(cov).splitlines()
         self.assertIn("mybranch.py 5 5 2 0 0%", report)
@@ -487,7 +508,7 @@ class SummaryTest(CoverageTest):
         """A helper for the next few tests."""
         cov = coverage.Coverage()
         cov.start()
-        import TheCode  # pragma: nested # pylint: disable=import-error
+        import TheCode  # pragma: nested # pylint: disable=import-error, unused-variable
         cov.stop()      # pragma: nested
         return self.get_report(cov)
 
@@ -522,7 +543,7 @@ class SummaryTest(CoverageTest):
             """)
         cov = coverage.Coverage()
         cov.start()
-        import start    # pragma: nested # pylint: disable=import-error
+        import start    # pragma: nested # pylint: disable=import-error, unused-variable
         cov.stop()      # pragma: nested
 
         report = self.get_report(cov)
@@ -542,7 +563,7 @@ class SummaryTest(CoverageTest):
         # Run the program.
         cov = coverage.Coverage()
         cov.start()
-        import main     # pragma: nested # pylint: disable=import-error
+        import main     # pragma: nested # pylint: disable=import-error, unused-variable
         cov.stop()      # pragma: nested
 
         report = self.get_report(cov).splitlines()
@@ -572,7 +593,7 @@ class SummaryTest(CoverageTest):
         # Run the program.
         cov = coverage.Coverage()
         cov.start()
-        import main     # pragma: nested # pylint: disable=import-error
+        import main     # pragma: nested # pylint: disable=import-error, unused-variable
         cov.stop()      # pragma: nested
 
         # Put back the missing Python file.
@@ -600,7 +621,7 @@ class SummaryTest2(CoverageTest):
         # statements, not one statement.
         cov = coverage.Coverage(branch=True)
         cov.start()
-        import usepkgs  # pragma: nested # pylint: disable=import-error
+        import usepkgs  # pragma: nested # pylint: disable=import-error, unused-variable
         cov.stop()      # pragma: nested
 
         repout = StringIO()
