@@ -8,11 +8,14 @@ import datetime
 import os
 import sys
 
+import pytest
+
 import coverage
 from coverage.backunittest import TestCase
 from coverage.files import actual_path
 
 from tests.coveragetest import CoverageTest
+from tests.helpers import CheckUniqueFilenames, re_lines, re_line
 
 
 class TestingTest(TestCase):
@@ -117,7 +120,7 @@ class CoverageTestTest(CoverageTest):
 
     def test_sub_python_is_this_python(self):
         # Try it with a Python command.
-        os.environ['COV_FOOBAR'] = 'XYZZY'
+        self.set_environ('COV_FOOBAR', 'XYZZY')
         self.make_file("showme.py", """\
             import os, sys
             print(sys.executable)
@@ -130,17 +133,77 @@ class CoverageTestTest(CoverageTest):
         self.assertEqual(out[2], 'XYZZY')
 
         # Try it with a "coverage debug sys" command.
-        out = self.run_command("coverage debug sys").splitlines()
-        # "environment: COV_FOOBAR = XYZZY" or "COV_FOOBAR = XYZZY"
-        executable = next(l for l in out if "executable:" in l)     # pragma: part covered
+        out = self.run_command("coverage debug sys")
+
+        executable = re_line(out, "executable:")
         executable = executable.split(":", 1)[1].strip()
-        self.assertTrue(same_python_executable(executable, sys.executable))
-        environ = next(l for l in out if "COV_FOOBAR" in l)         # pragma: part covered
+        self.assertTrue(_same_python_executable(executable, sys.executable))
+
+        # "environment: COV_FOOBAR = XYZZY" or "COV_FOOBAR = XYZZY"
+        environ = re_line(out, "COV_FOOBAR")
         _, _, environ = environ.rpartition(":")
         self.assertEqual(environ.strip(), "COV_FOOBAR = XYZZY")
 
 
-def same_python_executable(e1, e2):
+class CheckUniqueFilenamesTest(CoverageTest):
+    """Tests of CheckUniqueFilenames."""
+
+    run_in_temp_dir = False
+
+    class Stub(object):
+        """A stand-in for the class we're checking."""
+        def __init__(self, x):
+            self.x = x
+
+        def method(self, filename, a=17, b="hello"):
+            """The method we'll wrap, with args to be sure args work."""
+            return (self.x, filename, a, b)
+
+    def test_detect_duplicate(self):
+        stub = self.Stub(23)
+        CheckUniqueFilenames.hook(stub, "method")
+
+        # Two method calls with different names are fine.
+        assert stub.method("file1") == (23, "file1", 17, "hello")
+        assert stub.method("file2", 1723, b="what") == (23, "file2", 1723, "what")
+
+        # A duplicate file name trips an assertion.
+        with self.assertRaises(AssertionError):
+            stub.method("file1")
+
+
+@pytest.mark.parametrize("text, pat, result", [
+    ("line1\nline2\nline3\n", "line", "line1\nline2\nline3\n"),
+    ("line1\nline2\nline3\n", "[13]", "line1\nline3\n"),
+    ("line1\nline2\nline3\n", "X", ""),
+])
+def test_re_lines(text, pat, result):
+    assert re_lines(text, pat) == result
+
+@pytest.mark.parametrize("text, pat, result", [
+    ("line1\nline2\nline3\n", "line", ""),
+    ("line1\nline2\nline3\n", "[13]", "line2\n"),
+    ("line1\nline2\nline3\n", "X", "line1\nline2\nline3\n"),
+])
+def test_re_lines_inverted(text, pat, result):
+    assert re_lines(text, pat, match=False) == result
+
+@pytest.mark.parametrize("text, pat, result", [
+    ("line1\nline2\nline3\n", "2", "line2"),
+])
+def test_re_line(text, pat, result):
+    assert re_line(text, pat) == result
+
+@pytest.mark.parametrize("text, pat", [
+    ("line1\nline2\nline3\n", "line"),      # too many matches
+    ("line1\nline2\nline3\n", "X"),         # no matches
+])
+def test_re_line_bad(text, pat):
+    with pytest.raises(AssertionError):
+        re_line(text, pat)
+
+
+def _same_python_executable(e1, e2):
     """Determine if `e1` and `e2` refer to the same Python executable.
 
     Either path could include symbolic links.  The two paths might not refer
